@@ -191,6 +191,12 @@ options:
     scheduling_policy:
         description:
             - "Name of the scheduling policy to be used for cluster."
+    scheduling_policy_properties:
+        description:
+            - "Custom scheduling policy properties of the cluster."
+            - "These optional properties override the properties of the
+               scheduling policy specified by the C(scheduling_policy) parameter."
+        version_added: "2.6"
     cpu_arch:
         description:
             - "CPU architecture of cluster."
@@ -214,6 +220,19 @@ options:
             - "C(Note:)"
             - "This is supported since oVirt version 4.1."
         version_added: 2.4
+    external_network_providers:
+        description:
+            - "List of references to the external network providers available
+               in the cluster. If the automatic deployment of the external
+               network provider is supported, the networks of the referenced
+               network provider are available on every host in the cluster."
+            - "External network provider is described by following dictionary:"
+            - "C(name) - Name of the external network provider. Either C(name)
+               or C(id) is required."
+            - "C(id) - ID of the external network provider. Either C(name) or
+               C(id) is required."
+            - "This is supported since oVirt version 4.2."
+        version_added: 2.5
 extends_documentation_fragment: ovirt
 '''
 
@@ -250,6 +269,14 @@ EXAMPLES = '''
       - hwrng
       - random
 
+# Create cluster with default network provider
+- ovirt_cluster:
+    name: mycluster
+    data_center: Default
+    cpu_type: Intel SandyBridge Family
+    external_network_providers:
+      - name: ovirt-provider-ovn
+
 # Remove cluster
 - ovirt_cluster:
     state: absent
@@ -352,6 +379,20 @@ class ClustersModule(BaseModule):
 
         return mac_pool
 
+    def _get_external_network_providers(self):
+        return self.param('external_network_providers') or []
+
+    def _get_external_network_provider_id(self, external_provider):
+        return external_provider.get('id') or get_id_by_name(
+            self._connection.system_service().openstack_network_providers_service(),
+            external_provider.get('name')
+        )
+
+    def _get_external_network_providers_entity(self):
+        if self.param('external_network_providers') is not None:
+            return [otypes.ExternalProvider(id=self._get_external_network_provider_id(external_provider))
+                    for external_provider in self.param('external_network_providers')]
+
     def build_entity(self):
         sched_policy = self._get_sched_policy()
         return otypes.Cluster(
@@ -450,7 +491,9 @@ class ClustersModule(BaseModule):
                 name=self.param('network'),
             ) if self.param('network') else None,
             cpu=otypes.Cpu(
-                architecture=self.param('cpu_arch'),
+                architecture=otypes.Architecture(
+                    self.param('cpu_arch')
+                ) if self.param('cpu_arch') else None,
                 type=self.param('cpu_type'),
             ) if (
                 self.param('cpu_arch') or self.param('cpu_type')
@@ -465,17 +508,61 @@ class ClustersModule(BaseModule):
             mac_pool=otypes.MacPool(
                 id=get_id_by_name(self._connection.system_service().mac_pools_service(), self.param('mac_pool'))
             ) if self.param('mac_pool') else None,
+            external_network_providers=self._get_external_network_providers_entity(),
+            custom_scheduling_policy_properties=[
+                otypes.Property(
+                    name=sp.get('name'),
+                    value=str(sp.get('value')),
+                ) for sp in self.param('scheduling_policy_properties') if sp
+            ] if self.param('scheduling_policy_properties') is not None else None,
         )
 
+    def _matches_entity(self, item, entity):
+        return equal(item.get('id'), entity.id) and equal(item.get('name'), entity.name)
+
+    def _update_check_external_network_providers(self, entity):
+        if self.param('external_network_providers') is None:
+            return True
+        if entity.external_network_providers is None:
+            return not self.param('external_network_providers')
+        entity_providers = self._connection.follow_link(entity.external_network_providers)
+        entity_provider_ids = [provider.id for provider in entity_providers]
+        entity_provider_names = [provider.name for provider in entity_providers]
+        for provider in self._get_external_network_providers():
+            if provider.get('id'):
+                if provider.get('id') not in entity_provider_ids:
+                    return False
+            elif provider.get('name') and provider.get('name') not in entity_provider_names:
+                return False
+        for entity_provider in entity_providers:
+            if not any([self._matches_entity(provider, entity_provider)
+                        for provider in self._get_external_network_providers()]):
+                return False
+        return True
+
     def update_check(self, entity):
         sched_policy = self._get_sched_policy()
         migration_policy = getattr(entity.migration, 'policy', None)
+        cluster_cpu = getattr(entity, 'cpu', dict())
+
+        def check_custom_scheduling_policy_properties():
+            if self.param('scheduling_policy_properties'):
+                current = []
+                if entity.custom_scheduling_policy_properties:
+                    current = [(sp.name, str(sp.value)) for sp in entity.custom_scheduling_policy_properties]
+                passed = [(sp.get('name'), str(sp.get('value'))) for sp in self.param('scheduling_policy_properties') if sp]
+                for p in passed:
+                    if p not in current:
+                        return False
+            return True
+
         return (
+            check_custom_scheduling_policy_properties() and
             equal(self.param('comment'), entity.comment) and
             equal(self.param('description'), entity.description) and
             equal(self.param('switch_type'), str(entity.switch_type)) and
-            equal(self.param('cpu_arch'), str(entity.cpu.architecture)) and
-            equal(self.param('cpu_type'), entity.cpu.type) and
+            equal(self.param('cpu_arch'), str(getattr(cluster_cpu, 'architecture', None))) and
+            equal(self.param('cpu_type'), getattr(cluster_cpu, 'type', None)) and
             equal(self.param('ballooning'), entity.ballooning_enabled) and
             equal(self.param('gluster'), entity.gluster_service) and
             equal(self.param('virt'), entity.virt_service) and
@@ -515,7 +602,8 @@ class ClustersModule(BaseModule):
             equal(
                 get_id_by_name(self._connection.system_service().mac_pools_service(), self.param('mac_pool'), raise_error=False),
                 entity.mac_pool.id
-            )
+            ) and
+            self._update_check_external_network_providers(entity)
         )
 
 
@@ -536,7 +624,7 @@ def main():
         trusted_service=dict(default=None, type='bool'),
         vm_reason=dict(default=None, type='bool'),
         host_reason=dict(default=None, type='bool'),
-        memory_policy=dict(default=None, choices=['disabled', 'server', 'desktop']),
+        memory_policy=dict(default=None, choices=['disabled', 'server', 'desktop'], aliases=['performance_preset']),
         rng_sources=dict(default=None, type='list'),
         spice_proxy=dict(default=None),
         fence_enabled=dict(default=None, type='bool'),
@@ -564,6 +652,8 @@ def main():
         switch_type=dict(default=None, choices=['legacy', 'ovs']),
         compatibility_version=dict(default=None),
         mac_pool=dict(default=None),
+        external_network_providers=dict(default=None, type='list'),
+        scheduling_policy_properties=dict(type='list'),
     )
     module = AnsibleModule(
         argument_spec=argument_spec,
