@@ -21,8 +21,10 @@ __metaclass__ = type
 import os
 
 from ansible import constants as C
-from ansible.compat.six import string_types
-from ansible.errors import AnsibleParserError, AnsibleUndefinedVariable, AnsibleFileNotFound
+from ansible.errors import AnsibleParserError, AnsibleUndefinedVariable, AnsibleFileNotFound, AnsibleAssertionError
+from ansible.module_utils._text import to_native
+from ansible.module_utils.six import string_types
+from ansible.parsing.mod_args import ModuleArgsParser
 
 try:
     from __main__ import display
@@ -41,30 +43,43 @@ def load_list_of_blocks(ds, play, parent_block=None, role=None, task_include=Non
     # we import here to prevent a circular dependency with imports
     from ansible.playbook.block import Block
 
-    assert isinstance(ds, (list, type(None)))
+    if not isinstance(ds, (list, type(None))):
+        raise AnsibleAssertionError('%s should be a list or None but is %s' % (ds, type(ds)))
 
     block_list = []
     if ds:
-        for block_ds in ds:
-            b = Block.load(
-                block_ds,
-                play=play,
-                parent_block=parent_block,
-                role=role,
-                task_include=task_include,
-                use_handlers=use_handlers,
-                variable_manager=variable_manager,
-                loader=loader
-            )
+        count = iter(range(len(ds)))
+        for i in count:
+            block_ds = ds[i]
             # Implicit blocks are created by bare tasks listed in a play without
             # an explicit block statement. If we have two implicit blocks in a row,
             # squash them down to a single block to save processing time later.
-            if b._implicit and len(block_list) > 0 and block_list[-1]._implicit:
-                for t in b.block:
-                    t._block = block_list[-1]
-                block_list[-1].block.extend(b.block)
-            else:
-                block_list.append(b)
+            implicit_blocks = []
+            while block_ds is not None and not Block.is_block(block_ds):
+                implicit_blocks.append(block_ds)
+                i += 1
+                # Advance the iterator, so we don't repeat
+                next(count, None)
+                try:
+                    block_ds = ds[i]
+                except IndexError:
+                    block_ds = None
+
+            # Loop both implicit blocks and block_ds as block_ds is the next in the list
+            for b in (implicit_blocks, block_ds):
+                if b:
+                    block_list.append(
+                        Block.load(
+                            b,
+                            play=play,
+                            parent_block=parent_block,
+                            role=role,
+                            task_include=task_include,
+                            use_handlers=use_handlers,
+                            variable_manager=variable_manager,
+                            loader=loader,
+                        )
+                    )
 
     return block_list
 
@@ -84,11 +99,13 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
     from ansible.playbook.handler_task_include import HandlerTaskInclude
     from ansible.template import Templar
 
-    assert isinstance(ds, list)
+    if not isinstance(ds, list):
+        raise AnsibleAssertionError('The ds (%s) should be a list but was a %s' % (ds, type(ds)))
 
     task_list = []
     for task_ds in ds:
-        assert isinstance(task_ds, dict)
+        if not isinstance(task_ds, dict):
+            AnsibleAssertionError('The ds (%s) should be a dict but was a %s' % (ds, type(ds)))
 
         if 'block' in task_ds:
             t = Block.load(
@@ -103,7 +120,19 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
             )
             task_list.append(t)
         else:
-            if 'include' in task_ds:
+            args_parser = ModuleArgsParser(task_ds)
+            try:
+                (action, args, delegate_to) = args_parser.parse()
+            except AnsibleParserError as e:
+                # if the raises exception was created with obj=ds args, then it includes the detail
+                # so we dont need to add it so we can just re raise.
+                if e._obj:
+                    raise
+                # But if it wasn't, we can add the yaml object now to get more detail
+                raise AnsibleParserError(to_native(e), obj=task_ds, orig_exc=e)
+
+            if action in ('include', 'import_tasks', 'include_tasks'):
+
                 if use_handlers:
                     include_class = HandlerTaskInclude
                 else:
@@ -118,22 +147,31 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     loader=loader
                 )
 
-                all_vars = variable_manager.get_vars(loader=loader, play=play, task=t)
+                all_vars = variable_manager.get_vars(play=play, task=t)
                 templar = Templar(loader=loader, variables=all_vars)
 
                 # check to see if this include is dynamic or static:
                 # 1. the user has set the 'static' option to false or true
                 # 2. one of the appropriate config options was set
-                if t.static is not None:
+                if action == 'include_tasks':
+                    is_static = False
+                elif action == 'import_tasks':
+                    is_static = True
+                elif t.static is not None:
+                    display.deprecated("The use of 'static' has been deprecated. "
+                                       "Use 'import_tasks' for static inclusion, or 'include_tasks' for dynamic inclusion")
                     is_static = t.static
                 else:
                     is_static = C.DEFAULT_TASK_INCLUDES_STATIC or \
-                                (use_handlers and C.DEFAULT_HANDLER_INCLUDES_STATIC) or \
-                                (not templar._contains_vars(t.args['_raw_params']) and t.all_parents_static() and not t.loop)
+                        (use_handlers and C.DEFAULT_HANDLER_INCLUDES_STATIC) or \
+                        (not templar._contains_vars(t.args['_raw_params']) and t.all_parents_static() and not t.loop)
 
                 if is_static:
                     if t.loop is not None:
-                        raise AnsibleParserError("You cannot use 'static' on an include with a loop", obj=task_ds)
+                        if action == 'import_tasks':
+                            raise AnsibleParserError("You cannot use loops on 'import_tasks' statements. You should use 'include_tasks' instead.", obj=task_ds)
+                        else:
+                            raise AnsibleParserError("You cannot use 'static' on an include with a loop", obj=task_ds)
 
                     # we set a flag to indicate this include was static
                     t.statically_loaded = True
@@ -151,7 +189,19 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                         if not isinstance(parent_include, TaskInclude):
                             parent_include = parent_include._parent
                             continue
-                        parent_include_dir = templar.template(os.path.dirname(parent_include.args.get('_raw_params')))
+                        try:
+                            parent_include_dir = os.path.dirname(templar.template(parent_include.args.get('_raw_params')))
+                        except AnsibleUndefinedVariable as e:
+                            if not parent_include.statically_loaded:
+                                raise AnsibleParserError(
+                                    "Error when evaluating variable in dynamic parent include path: %s. "
+                                    "When using static imports, the parent dynamic include cannot utilize host facts "
+                                    "or variables from inventory" % parent_include.args.get('_raw_params'),
+                                    obj=task_ds,
+                                    suppress_extended_error=True,
+                                    orig_exc=e
+                                )
+                            raise
                         if cumulative_path is None:
                             cumulative_path = parent_include_dir
                         elif not os.path.isabs(cumulative_path):
@@ -172,15 +222,15 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     if not found:
                         try:
                             include_target = templar.template(t.args['_raw_params'])
-                        except AnsibleUndefinedVariable:
+                        except AnsibleUndefinedVariable as e:
                             raise AnsibleParserError(
-                                      "Error when evaluating variable in include name: %s.\n\n" \
-                                      "When using static includes, ensure that any variables used in their names are defined in vars/vars_files\n" \
-                                      "or extra-vars passed in from the command line. Static includes cannot use variables from inventory\n" \
-                                      "sources like group or host vars." % t.args['_raw_params'],
-                                      obj=task_ds,
-                                      suppress_extended_error=True,
-                                  )
+                                "Error when evaluating variable in import path: %s.\n\n"
+                                "When using static imports, ensure that any variables used in their names are defined in vars/vars_files\n"
+                                "or extra-vars passed in from the command line. Static imports cannot use variables from facts or inventory\n"
+                                "sources like group or host vars." % t.args['_raw_params'],
+                                obj=task_ds,
+                                suppress_extended_error=True,
+                                orig_exc=e)
                         if t._role:
                             include_file = loader.path_dwim_relative(t._role._role_path, subdir, include_target)
                         else:
@@ -189,7 +239,8 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     try:
                         data = loader.load_from_file(include_file)
                         if data is None:
-                            return []
+                            display.warning('file %s is empty and had no tasks to include' % include_file)
+                            continue
                         elif not isinstance(data, list):
                             raise AnsibleParserError("included task files must contain a list of tasks", obj=data)
 
@@ -197,52 +248,58 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                         # the same fashion used by the on_include callback. We also do it here,
                         # because the recursive nature of helper methods means we may be loading
                         # nested includes, and we want the include order printed correctly
-                        display.vv("statically included: %s" % include_file)
+                        display.vv("statically imported: %s" % include_file)
                     except AnsibleFileNotFound:
-                        if t.static or \
+                        if action != 'include' or t.static or \
                            C.DEFAULT_TASK_INCLUDES_STATIC or \
                            C.DEFAULT_HANDLER_INCLUDES_STATIC and use_handlers:
                             raise
                         display.deprecated(
-                            "Included file '%s' not found, however since this include is not " \
-                            "explicitly marked as 'static: yes', we will try and include it dynamically " \
-                            "later. In the future, this will be an error unless 'static: no' is used " \
-                            "on the include task. If you do not want missing includes to be considered " \
-                            "dynamic, use 'static: yes' on the include or set the global ansible.cfg " \
-                            "options to make all inclues static for tasks and/or handlers" % include_file,
+                            "Included file '%s' not found, however since this include is not "
+                            "explicitly marked as 'static: yes', we will try and include it dynamically "
+                            "later. In the future, this will be an error unless 'static: no' is used "
+                            "on the include task. If you do not want missing includes to be considered "
+                            "dynamic, use 'static: yes' on the include or set the global ansible.cfg "
+                            "options to make all includes static for tasks and/or handlers" % include_file, version="2.7"
                         )
                         task_list.append(t)
                         continue
 
+                    ti_copy = t.copy(exclude_parent=True)
+                    ti_copy._parent = block
                     included_blocks = load_list_of_blocks(
                         data,
                         play=play,
                         parent_block=None,
-                        task_include=t.copy(),
+                        task_include=ti_copy,
                         role=role,
                         use_handlers=use_handlers,
                         loader=loader,
                         variable_manager=variable_manager,
                     )
 
+                    # FIXME: remove once 'include' is removed
                     # pop tags out of the include args, if they were specified there, and assign
                     # them to the include. If the include already had tags specified, we raise an
                     # error so that users know not to specify them both ways
-                    tags = t.vars.pop('tags', [])
+                    tags = ti_copy.vars.pop('tags', [])
                     if isinstance(tags, string_types):
                         tags = tags.split(',')
 
                     if len(tags) > 0:
-                        if len(t.tags) > 0:
+                        if action in ('include_tasks', 'import_tasks'):
+                            raise AnsibleParserError('You cannot specify "tags" inline to the task, it is a task keyword')
+                        if len(ti_copy.tags) > 0:
                             raise AnsibleParserError(
-                                "Include tasks should not specify tags in more than one way (both via args and directly on the task). " \
+                                "Include tasks should not specify tags in more than one way (both via args and directly on the task). "
                                 "Mixing styles in which tags are specified is prohibited for whole import hierarchy, not only for single import statement",
                                 obj=task_ds,
                                 suppress_extended_error=True,
                             )
-                        display.deprecated("You should not specify tags in the include parameters. All tags should be specified using the task-level option")
+                        display.deprecated("You should not specify tags in the include parameters. All tags should be specified using the task-level option",
+                                           version="2.7")
                     else:
-                        tags = t.tags[:]
+                        tags = ti_copy.tags[:]
 
                     # now we extend the tags on each of the included blocks
                     for b in included_blocks:
@@ -257,44 +314,52 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     else:
                         task_list.extend(included_blocks)
                 else:
+                    t.is_static = False
                     task_list.append(t)
 
-            elif 'include_role' in task_ds:
-
+            elif action in ('include_role', 'import_role'):
                 ir = IncludeRole.load(
-                            task_ds,
-                            block=block,
-                            role=role,
-                            task_include=None,
-                            variable_manager=variable_manager,
-                            loader=loader
-                     )
+                    task_ds,
+                    block=block,
+                    role=role,
+                    task_include=None,
+                    variable_manager=variable_manager,
+                    loader=loader,
+                )
 
                 #   1. the user has set the 'static' option to false or true
                 #   2. one of the appropriate config options was set
-                if ir.static is not None:
+                is_static = False
+                if action == 'import_role':
+                    is_static = True
+
+                elif ir.static is not None:
+                    display.deprecated("The use of 'static' for 'include_role' has been deprecated. "
+                                       "Use 'import_role' for static inclusion, or 'include_role' for dynamic inclusion")
                     is_static = ir.static
-                else:
-                    display.debug('Determine if include_role is static')
-                    # Check to see if this include is dynamic or static:
-                    all_vars = variable_manager.get_vars(loader=loader, play=play, task=ir)
-                    templar = Templar(loader=loader, variables=all_vars)
-                    needs_templating = False
-                    for param in ir.args:
-                        if templar._contains_vars(ir.args[param]):
-                            if not templar.templatable(ir.args[param]):
-                                needs_templating = True
-                                break
-                    is_static = C.DEFAULT_TASK_INCLUDES_STATIC or \
-                                (use_handlers and C.DEFAULT_HANDLER_INCLUDES_STATIC)  or \
-                                (not needs_templating and ir.all_parents_static() and not ir.loop)
-                    display.debug('Determined that if include_role static is %s' % str(is_static))
+
                 if is_static:
+                    if ir.loop is not None:
+                        if action == 'import_role':
+                            raise AnsibleParserError("You cannot use loops on 'import_role' statements. You should use 'include_role' instead.", obj=task_ds)
+                        else:
+                            raise AnsibleParserError("You cannot use 'static' on an include_role with a loop", obj=task_ds)
+
+                    # we set a flag to indicate this include was static
+                    ir.statically_loaded = True
+
+                    # template the role name now, if needed
+                    all_vars = variable_manager.get_vars(play=play, task=ir)
+                    templar = Templar(loader=loader, variables=all_vars)
+                    if templar._contains_vars(ir._role_name):
+                        ir._role_name = templar.template(ir._role_name)
+
                     # uses compiled list from object
-                    t = task_list.extend(ir.get_block_list(variable_manager=variable_manager, loader=loader))
+                    blocks, _ = ir.get_block_list(variable_manager=variable_manager, loader=loader)
+                    task_list.extend(blocks)
                 else:
                     # passes task object itself for latter generation of list
-                    t = task_list.append(ir)
+                    task_list.append(ir)
             else:
                 if use_handlers:
                     t = Handler.load(task_ds, block=block, role=role, task_include=task_include, variable_manager=variable_manager, loader=loader)
@@ -315,7 +380,8 @@ def load_list_of_roles(ds, play, current_role_path=None, variable_manager=None,
     # we import here to prevent a circular dependency with imports
     from ansible.playbook.role.include import RoleInclude
 
-    assert isinstance(ds, list)
+    if not isinstance(ds, list):
+        raise AnsibleAssertionError('ds (%s) should be a list but was a %s' % (ds, type(ds)))
 
     roles = []
     for role_def in ds:
@@ -323,4 +389,3 @@ def load_list_of_roles(ds, play, current_role_path=None, variable_manager=None,
         roles.append(i)
 
     return roles
-
